
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <unistd.h>
#include <stdio.h>
#include "deditec_msgqueue.h"

int dt_msgq_create (int permission)
{
	return msgget(IPC_PRIVATE, permission | IPC_CREAT);
}

// ---------------------------------------------------------------

void dt_msgq_delete (int id)
{
	msgctl(id, IPC_RMID, 0);
}

// ---------------------------------------------------------------

unsigned long dt_msgq_get (int id, void *retstr, unsigned long retstr_len)
{
	return (msgrcv(id, retstr, retstr_len, 0, 0) == -1) ? 1 : 0;
}

// ---------------------------------------------------------------

unsigned long dt_msgq_put (int id, void *sendstr, unsigned long sendstr_len)
{
	return (msgsnd(id, sendstr, sendstr_len, 0) == -1) ? 1 : 0;
}

// ---------------------------------------------------------------

unsigned long dt_msgq_global_exists (char* filename)
{
	return (access(filename, F_OK) == 0) ? 0 : 1;
}

// ---------------------------------------------------------------

unsigned long dt_msgq_global_create (int id, char* filename)
{
	FILE *fh = fopen(filename, "w+");
	if (!fh)
	{
		return 1;
	}
	if (!fwrite(&id, sizeof(int), 1, fh))
	{
		fclose(fh);
		return 1;
	}
	fclose(fh);
	chmod(filename, 644);
	return 0;
}

// ---------------------------------------------------------------

int dt_msgq_global_read (char* filename)
{
	int id;
	FILE *fh;
	while (dt_msgq_global_exists(filename) != 0)
	{
		usleep(500);
	}
	while (!(fh = fopen(filename, "r")))
	{
		usleep(500);
	}
	if (!fread(&id, sizeof(int), 1, fh))
	{
		fclose(fh);
		return -1;
	}
	fclose(fh);
	return id;
}

// ---------------------------------------------------------------

void dt_msgq_global_delete (char* filename)
{
	remove(filename);
}

// ---------------------------------------------------------------
