#include <stdio.h>
#include <string.h>

#include "deditec_parameter_io.h"
#include "deditec_msgqueue.h"
#include "deditec_msgqueue_struct.h"
#include "dt_embedded_parameter.h"


// C-Datei Include - Zur einfacheren Einbindung in alte Projekte
// #include "deditec_msgqueue.c" // sollte in den compiler aufgenommen werden


#define RET_SUCCESS 0
#define RET_ERROR 1
#define RET_MALFORMED 2
#define RET_COMMUNICATIONISSUE 3

// #define DEBUG_COMMANDS 1							// if defined, prints debug outputs
// #define LOG_WRITE_ENABLE 1							// set to 1 to log debug messages, else 0

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

extern unsigned long parameter_delete (unsigned char *pattern, unsigned long pattern_length)
{
	int serverid, clientid;
	dt_msgq_c2s putdata;
	dt_msgq_s2c getdata;

	putdata.content.cmd = 'D';

	if (pattern_length > 63)
	{
		return RET_ERROR;
	}

	putdata.content.keylen = (unsigned char)pattern_length;
	strcpy_n(putdata.content.key, 64, pattern, pattern_length);

	// ---------------------------------------------------

	serverid = dt_msgq_global_read(DT_MSGQ_PARAM);
	if (serverid < 0)
	{
		// printf("cannot read serverid\n");
		return RET_COMMUNICATIONISSUE;
	}
	clientid = dt_msgq_create(DT_MSGQ_PERM);
	if (clientid < 0)
	{
		// printf("cannot create client queue\n");
		return RET_COMMUNICATIONISSUE;
	}

	putdata.priority = 1;
	putdata.content.clientid = clientid;

	// ---------------------------------------------------

	if (dt_msgq_put(serverid, &putdata, sizeof(dt_msgq_c2s_s)) != 0)
	{
		// printf("FATAL ERROR: in dt_msgq_put failed to put a queue message\n");
		dt_msgq_delete(clientid);
		return RET_COMMUNICATIONISSUE;
	}

	// printf("client waits for server response\n");

	if (dt_msgq_get(clientid, &getdata, sizeof(dt_msgq_s2c_s)) != 0)
	{
		// printf("FATAL ERROR: in dt_msgq_get failed to get a queue message\n");
		dt_msgq_delete(clientid);
		return RET_COMMUNICATIONISSUE;
	}

	if (getdata.content.status != 0)
	{
		// printf("server failed to delete that key\n");
		dt_msgq_delete(clientid);
		return RET_ERROR;
	}

	dt_msgq_delete(clientid);

	return RET_SUCCESS;
}
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_text (unsigned char *pattern, unsigned long pattern_length, unsigned char *found_value_buff, unsigned long found_value_buff_size)
{
	int serverid, clientid;
	dt_msgq_c2s putdata;
	dt_msgq_s2c getdata;
	
	*found_value_buff = 0;

	putdata.content.cmd = 'R';

	if (pattern_length > 63)
	{
		return RET_ERROR;
	}

	putdata.content.keylen = (unsigned char)pattern_length;
	strcpy_n(putdata.content.key, 64, pattern, pattern_length);

	// ---------------------------------------------------

	serverid = dt_msgq_global_read(DT_MSGQ_PARAM);
	if (serverid < 0)
	{
		// printf("cannot read serverid\n");
		return RET_COMMUNICATIONISSUE;
	}
	clientid = dt_msgq_create(DT_MSGQ_PERM);
	if (clientid < 0)
	{
		// printf("cannot create client queue\n");
		return RET_COMMUNICATIONISSUE;
	}

	putdata.priority = 1;
	putdata.content.clientid = clientid;

	// ---------------------------------------------------

	if (dt_msgq_put(serverid, &putdata, sizeof(dt_msgq_c2s_s)) != 0)
	{
		// printf("FATAL ERROR: in dt_msgq_put failed to put a queue message\n");
		dt_msgq_delete(clientid);
		return RET_COMMUNICATIONISSUE;
	}

	// printf("client waits for server response\n");

	if (dt_msgq_get(clientid, &getdata, sizeof(dt_msgq_s2c_s)) != 0)
	{
		// printf("FATAL ERROR: in dt_msgq_get failed to get a queue message\n");
		dt_msgq_delete(clientid);
		return RET_COMMUNICATIONISSUE;
	}

	if (getdata.content.status != 0)
	{
		// printf("server failed to get that key\n");
		dt_msgq_delete(clientid);
		return RET_ERROR;
	}

	// printf("val: %s\n", getdata.content.value);
	strcpy_n(found_value_buff, found_value_buff_size, getdata.content.value, getdata.content.vallen);
	// printf("val: %s, strlen: %i, vallen: %u\n", found_value_buff, strlen(found_value_buff), getdata.content.vallen);
	dt_msgq_delete(clientid);

	return RET_SUCCESS;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_long (unsigned char *pattern, unsigned long pattern_length, long *found_value)
{
	unsigned char buff[64];
	unsigned long pos = 0;
	long retval = 0;
	unsigned long negative = 0;
	unsigned long returned_value;

	returned_value = parameter_get_text (pattern, pattern_length, buff, 64);

	if (RET_SUCCESS != returned_value)
	{
		return returned_value;
	}

	if (buff[0] == '-')
	{
		negative = 1;
		pos++;
	}
	else if (buff[0] == '+')
	{
		pos++;
	}

	for (; pos < (19 - negative); pos++)
	{
		if (buff[pos] < '0' || buff[pos] > '9')
		{
			if (buff[pos] == 0)
			{
				break;
			}
			else
			{
				return RET_MALFORMED;
			}
		}

		retval = retval * 10 + buff[pos] - '0';
	}

	if (negative)
	{
		*found_value = retval * -1;
		return RET_SUCCESS;
	}

	*found_value = retval;
	return RET_SUCCESS;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_get_float (unsigned char *pattern, unsigned long pattern_length, float *return_value)
{
	unsigned char buff[64];
	unsigned long returned_value;

	returned_value = parameter_get_text (pattern, pattern_length, buff, 64);

	if (RET_SUCCESS != returned_value)
	{
		return returned_value;
	}

	*return_value = (float)atof(buff);

	return RET_SUCCESS;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_set_text (unsigned char *pattern, unsigned long pattern_length, unsigned char *insert_value, unsigned long insert_value_length)
{
	int serverid, clientid;
	dt_msgq_c2s putdata;
	dt_msgq_s2c getdata;

	putdata.content.cmd = 'W';
	putdata.content.keylen = pattern_length > 64 ? 64 : pattern_length;
	putdata.content.vallen = insert_value_length > 64 ? 64 : insert_value_length;

	strcpy_n(putdata.content.key, 64, pattern, pattern_length);
	strcpy_n(putdata.content.value, 64, insert_value, insert_value_length);

	// ---------------------------------------------------

	serverid = dt_msgq_global_read(DT_MSGQ_PARAM);
	if (serverid < 0)
	{
		// printf("cannot read serverid\n");
		return RET_COMMUNICATIONISSUE;
	}

	clientid = dt_msgq_create(DT_MSGQ_PERM);

	if (clientid < 0)
	{
		// printf("cannot create client queue\n");
		return RET_COMMUNICATIONISSUE;
	}

	putdata.priority = 1;
	putdata.content.clientid = clientid;

	// ---------------------------------------------------

	if (dt_msgq_put(serverid, &putdata, sizeof(dt_msgq_c2s_s)) != 0)
	{
		// printf("FATAL ERROR: in dt_msgq_put failed to put a queue message\n");
		dt_msgq_delete(clientid);
		return RET_COMMUNICATIONISSUE;
	}

	if (dt_msgq_get(clientid, &getdata, sizeof(dt_msgq_s2c_s)) != 0)
	{
		// printf("FATAL ERROR: in dt_msgq_get failed to get a queue message\n");
		dt_msgq_delete(clientid);
		return RET_COMMUNICATIONISSUE;
	}

	if (getdata.content.status != 0)
	{
		dt_msgq_delete(clientid);
		return RET_ERROR;
	}

	dt_msgq_delete(clientid);
	return RET_SUCCESS;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_set_long (unsigned char *pattern, unsigned long pattern_length, long insert_value)
{
	char insert_value_c[64];
	unsigned long returned_value;

	sprintf(insert_value_c, "%ld", insert_value);

	returned_value = parameter_set_text(pattern, pattern_length, (unsigned char*)insert_value_c, strlen(insert_value_c));

	if (RET_SUCCESS != returned_value)
	{
		return returned_value;
	}
	return RET_SUCCESS;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long parameter_set_float (unsigned char *pattern, unsigned long pattern_length, float insert_value)
{
	char insert_value_c[64];
	unsigned long returned_value;

	sprintf(insert_value_c, "%f", insert_value);

	returned_value = parameter_set_text(pattern, pattern_length, (unsigned char*)insert_value_c, strlen(insert_value_c));

	if (RET_SUCCESS != returned_value)
	{
		return returned_value;
	}
	return RET_SUCCESS;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
