//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
//  product: ro-eth
//  configuration: custom_sample for RO-ETH-Modules
//  programming language: vc
//
//
//  (c) DEDITEC GmbH, 2012
//  web: http://www.deditec.de/
//  mail: vertrieb@deditec.de
//
// 
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************


#include <stdio.h>
#include <stdlib.h>  // usefull for exit(), atoi()
#include <string.h>  // bzero, strlen, strcpy, bcopy

#ifndef __unix__        // set include for windows
#include <malloc.h>
#endif

#include "../library/vc/deditec_tcp_io.h"
#include "../library/vc/deditec_string_utils.h"
#include "../library/vc/deditec_string_convert_to_number.h"
#include "../library/vc/deditec_parameter_io.h"
#include "../library/vc/deditec_log.h"

#include "../library/vc/dt_embedded_parameter.h"
#include "../library/vc/dt_communication_paramams.h"


#include "../delib-linux/delib-sources/delib/delib.h"

#include "proj_config.h"


#define bool unsigned char
#define false 1
#define true 0

extern char global_ip_addr[];



//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************


// ---------------------------------------------------------------------------
// GetLastError function

bool IsError()
{
	unsigned char msg_error[500];
	char msg[500];

	if (DapiGetLastError() != DAPI_ERR_NONE)
	{

		DapiGetLastErrorText((unsigned char*) msg_error, sizeof(msg_error));
		sprintf(msg, "Error Code = %x * Message = %s\n", 0, msg_error);
		log_write(msg, 1);
		

		DapiClearLastError();

		return true;
	}

	return false;
}


//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************


int main(int argc, char *argv[])
{

	char test_text_variable[20];
	unsigned long test_long_variable;
	unsigned long handle;
	unsigned long data;
	float ad_da_data;
	int ret;
	char msg[200];


	// --------------------------------------------------------------------
	// LOG Init
	log_init("CUST-FW", 1);				// Init Log with pre-text and enable global output
	
	sprintf(msg, "Starting custom1 firmware");		// Schreibt einen beliebigen String in den buffer "msg"
	log_write(msg, 1);								// Schreibt den buffer "msg" in LOG * Details: siehe "/library/vc/deditec_log.h"
	// -------------------------------------------------------------------
	
	
	// --------------------------------------------------------------------
	// Setzen der Version Nr.
	if(versionNr_0 == '0')
	{
		sprintf(msg, "%c.%c%c", versionNr_1, versionNr_2, versionNr_3);		// Versionsnummer: siehe "proj_config.h"
	}
	else
	{
		sprintf(msg, "%c%c.%c%c", versionNr_0, versionNr_1, versionNr_2, versionNr_3);
	}
	parameter_set_text(DEDITEC_PARAM_FW_REV_CUSTOM1, strlen(DEDITEC_PARAM_FW_REV_CUSTOM1), msg, strlen(msg));							// Schreibt Variable ("Firmware-Version" @ Weboberflche)
	// --------------------------------------------------------------------
	
	
	
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	// Lesen/Schreiben der Variablen (defines, siehe datei "proj_config.h") + Ausgabe im LOG * Diese Variablen werden auch automatisch im Custom-Bereich der WEB-Oberflche angezeigt
	
	
	
	// --------------------------------------------------------------------
	// Schreiben einer Variablen 
	parameter_set_text(CUSTOM1_PARAM_PRODUCT_NAME, strlen(CUSTOM1_PARAM_PRODUCT_NAME), "MyCustomProject", strlen("MyCustomProject"));	// Schreibt Variable ("Projektname" @ Weboberflche)
	// --------------------------------------------------------------------
	
	
	// --------------------------------------------------------------------
	// Lesen der Variablen
	ret = parameter_get_text(CUSTOM1_TEST_PARAM_TEXT, strlen(CUSTOM1_TEST_PARAM_TEXT), test_text_variable, strlen(test_text_variable)); // Liest Variable ("Test Variable 1" @ Weboberflche)
	ret = parameter_get_long(CUSTOM1_TEST_PARAM_LONG, strlen(CUSTOM1_TEST_PARAM_LONG), &test_long_variable);							// Liest Variable ("Test Variable 2" @ Weboberflche)
	// --------------------------------------------------------------------
	
	
	// --------------------------------------------------------------------
	// Ausgabe der gelesenen Variablen ins LOG
	sprintf(msg, "Test Variable 1 = %s\n", test_text_variable);
	log_write(msg, 1);
	
	sprintf(msg, "Test Variable 2 = %d\n", test_long_variable);
	log_write(msg, 1);
	// --------------------------------------------------------------------
	
	
	
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	// RO-ETH-Modul ffnen + AD/DA Modi setzen
	
	
	
	// --------------------------------------------------------------------
	// DEDITEC RO-ETH Modul ffnen * jede Sekunde retry + Ausgabe im LOG
	ret = parameter_get_text(DEDITEC_PARAM_IP_IP,  strlen(DEDITEC_PARAM_IP_IP), global_ip_addr, 20);										// Auslesen der RO-ETH-IP-Adresse
	sprintf(msg, "Using internal IP: %s\n", global_ip_addr);
	log_write(msg, 1);
	
	do 
	{
		handle = DapiOpenModule(RO_ETH, 0);
		IsError();
		if(handle == 0)
		{
			sprintf(msg, "Error OpenModule - Handle=0\n");
			log_write(msg, 1);
		
			usleep(1000*1000);		// 1 sec retry
		}
	}
	while(handle == 0);

	sprintf(msg, "RO-ETH Module is open\n");
	log_write(msg, 1);
	// --------------------------------------------------------------------
	
	// --------------------------------------------------------------------
	// AD/DA Modi setzen
	DapiDASetMode(handle, 0, ADDA_MODE_BIPOL_10V);	// Setzt analogen Ausgangskanal 0 auf den Bereich 0V .. 10V
	IsError();
	DapiDASetMode(handle, 1, ADDA_MODE_BIPOL_10V);	// Setzt analogen Ausgangskanal 1 auf den Bereich 0V .. 10V
	IsError();
	DapiDASetMode(handle, 2, ADDA_MODE_BIPOL_10V);	// Setzt analogen Ausgangskanal 2 auf den Bereich 0V .. 10V
	IsError();
	DapiDASetMode(handle, 3, ADDA_MODE_BIPOL_10V);	// Setzt analogen Ausgangskanal 3 auf den Bereich 0V .. 10V
	IsError();

	DapiADSetMode(handle, 0, ADDA_MODE_BIPOL_10V);	// Setzt analogen Eingangskanal 0 auf den Bereich 0V .. 10V
	IsError();
	DapiADSetMode(handle, 1, ADDA_MODE_BIPOL_10V);	// Setzt analogen Eingangskanal 1 auf den Bereich 0V .. 10V
	IsError();
	DapiADSetMode(handle, 2, ADDA_MODE_BIPOL_10V);	// Setzt analogen Eingangskanal 2 auf den Bereich 0V .. 10V
	IsError();
	DapiADSetMode(handle, 3, ADDA_MODE_BIPOL_10V);	// Setzt analogen Eingangskanal 3 auf den Bereich 0V .. 10V
	IsError();
	// --------------------------------------------------------------------
	
	
	// --------------------------------------------------------------------
	// AD Eingnge lesen
	ad_da_data = DapiADGetVolt(handle, 0);	// Liest analogen Eingangskanal 0
	IsError();
	sprintf(msg, "Reading analogue input 0 * data = %f\n", ad_da_data);
	log_write(msg, 1);
	
	ad_da_data = DapiADGetVolt(handle, 1);	// Liest analogen Eingangskanal 1
	IsError();
	sprintf(msg, "Reading analogue input 1 * data = %f\n", ad_da_data);
	log_write(msg, 1);
	
	ad_da_data = DapiADGetVolt(handle, 2);	// Liest analogen Eingangskanal 2
	IsError();
	sprintf(msg, "Reading analogue input 2 * data = %f\n", ad_da_data);
	log_write(msg, 1);
	// --------------------------------------------------------------------
	
	
	// --------------------------------------------------------------------
	// DA Ausgng schreiben
	ad_da_data = 5.00;
	DapiDASetVolt(handle, 0, ad_da_data);		// Schreibt 5V auf den analogen Ausgangskanal 0
	IsError();
	sprintf(msg, "Setting analogue output 0 to %fV\n", ad_da_data);
	log_write(msg, 1);
	
	ad_da_data = 7.50;
	DapiDASetVolt(handle, 0, ad_da_data);		// Schreibt 7,5V auf den analogen Ausgangskanal 0
	IsError();
	sprintf(msg, "Setting analogue output 1 to %fV\n", ad_da_data);
	log_write(msg, 1);		
	// --------------------------------------------------------------------
	
	
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	//-----------------------------------------------------------------------------
	// Endlos-Schleife, die permanent die ersten 8 digitalen Eingnge auf die ersten 8 digitalen Ausgnge umsetzt.
	
	do 
	{
		// --------------------------------------------------------------------
		// digitale Eingnge lesen + Ausgabe im LOG
		data = DapiDIGet8(handle, 0);				// Liest digitale Eingangskanle 0-7
		IsError();
		sprintf(msg, "Reading digital input 0-7 * data = 0x%x\n", data);
		log_write(msg, 1);
		//-----------------------------------------------------------------------------
	
		// --------------------------------------------------------------------
		// digitale Ausgnge setzen + Ausgabe im LOG
		DapiDOSet8(handle, 0, data);				// Setzt digitale Ausgangskanle 0-7
		IsError();
		sprintf(msg, "Writing this data to digital output 0-7\n");
		log_write(msg, 1);
		// --------------------------------------------------------------------
	

		usleep(1*1000*1000);									// 1 sec Verzgerung
	}
	while (1==1); // Endlos-Schleife
	
} // main
	
	
	
	
	

	
